# =====================================================================
# AWK functions for CSA.
# 
# Copyright (c) 2001,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
# 
# This program is distributed in the hope that it will be useful,  
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: BEGIN.awk,v 1.3 2006/03/10 21:36:17 carlo Exp $

BEGIN {

  # The CMD global variable is used to hold system commands. It is
  # a global name because AWK can have only one system command opened
  # at a time, and the current command needs to be closed before a new
  # one can be opened.

  _NULL = _RESULT = _CMD = ""

  _TRUE = 0; _FALSE = 1

  _RDFPRINT = 1

  _BASE64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"

  _MSG[0] = 0				# message array.

  # Generic error code.
  _EFAULT	= 255

  # Linux-style symbolic error codes (see errno(3) and <errno.h>).
  _ENOENT	=   2
  _EIO		=   5
  _EACCES	=  13
  _EISDIR	=  21
  _EINVAL	=  22
  _ENOMSG	=  42
  _EMSGSIZE	=  90

  # Various flags and misc. names.
  _O_TRUNC = _O_REQUEST = _ERRNO = 0
  _O_APPEND = _O_RESPONSE = _O_MIDDLE = _O_NOENC = 1
  _O_PATHINFO = _O_CURR = 1
  _O_ATTACH = 2

  # Get local settings.
  _CSAINST = ENVIRON["CSA_INSTALL"]
  _STDOUT = ENVIRON["CSA_STDOUT"]
  _STDERR = ENVIRON["CSA_STDERR"]

  # Set default values if necessary.
  if (_CSAINST !~ /^\//) _CSAINST = "/usr/local/csa"
  if (_STDOUT !~ /^\//) _STDOUT = "/dev/stdout"
  if (_STDERR !~ /^\//) _STDERR = "/dev/stderr"

  # Set local time values if available.
  _TIME["Y"] = _rcget("CSA_TIME_LOCAL",1)
  _TIME["m"] = _rcget("CSA_TIME_LOCAL",2)
  _TIME["d"] = _rcget("CSA_TIME_LOCAL",3)
  _TIME["H"] = _rcget("CSA_TIME_LOCAL",4)
  _TIME["M"] = _rcget("CSA_TIME_LOCAL",5)
  _TIME["S"] = _rcget("CSA_TIME_LOCAL",6)
  _TIME["Z"] = _rcget("CSA_TIME_LOCAL",7)
  _TIME["a"] = _rcget("CSA_TIME_LOCAL",8)
  _TIME["b"] = _rcget("CSA_TIME_LOCAL",9)
  _TIME["s"] = _rcget("CSA_TIME_LOCAL",10)
  _TIME["z"] = _rcget("CSA_TIME_LOCAL",11)

  # Set UTC time values if available.
  _UTIME["Y"] = _rcget("CSA_TIME_UTC",1)
  _UTIME["m"] = _rcget("CSA_TIME_UTC",2)
  _UTIME["d"] = _rcget("CSA_TIME_UTC",3)
  _UTIME["H"] = _rcget("CSA_TIME_UTC",4)
  _UTIME["M"] = _rcget("CSA_TIME_UTC",5)
  _UTIME["S"] = _rcget("CSA_TIME_UTC",6)
  _UTIME["Z"] = _rcget("CSA_TIME_UTC",7)
  _UTIME["a"] = _rcget("CSA_TIME_UTC",8)
  _UTIME["b"] = _rcget("CSA_TIME_UTC",9)
  _UTIME["s"] = _rcget("CSA_TIME_UTC",10)
  _UTIME["z"] = _rcget("CSA_TIME_UTC",11)
}

