# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# int _envtoxml([string template, string fileroot, string eval,\
#		string table, boolean trim, string tago, string tagc,\
#		string ofile, int lvl, boolean parse,\
#		boolean norepl, string prg])
#
# Take environment variables and use them to replace special tags in
# an XML template file. All arguments are optional, but at least the
# 'template' parameter should be passed, otherwise the result will
# hardly be of any usefulness :-)
# =====================================================================

function _envtoxml(tpl,froot,eval,tbl,trim,tago,tagc,ofile,lvl,parse,\
		   norepl,prg,		ltago,ltagc,retago,retagc,nl,\
					end,tlen,tag,enc,a,e,i,j,t,T,\
					tb,input,record,start,value) {
  i=j=0; norepl/=1

  # Check args and set defaults.
  if (tpl == "") tpl = "/dev/null"
  if (ofile == "") ofile = _STDOUT
  if (froot == "") froot = ENVIRON["CSA_ROOT"] "/xml/"
  if (tago == "") tago = "$["
  if (tagc == "") tagc = "]"
  if (tago == tagc) return _EINVAL # start- and end-tags must be different
  if (!ENVIRON["CSA_TPLMAX"]) ENVIRON["CSA_TPLMAX"] = 10
  if (_isuri(tolower(prg)) == _FALSE) prg = _NULL

  # prevent infinite recursions.
  if (lvl > ENVIRON["CSA_TPLMAX"]) return 1

  ltago = length(tago)
  ltagc = length(tagc)

  # Prepare the substitution regexp's.
  retago = _escreg(tago)
  retagc = _escreg(tagc)

  # tags can be expressed in several formats, including:
  #
  # 1) CSA standard (canonical) format:
  #
  #    $[tag:f]
  #
  #    where '$[' and ']' can be replaced with different sequences,
  #    and 'f' is the encoding flag, which is used only in external
  #    file inclusions.
  #
  # 2) XML format:
  #
  #    <csa:tag enc="f" />
  #
  #    Unfortunately the XML format is of very limited use, as it
  #    breaks the template well-formedness when used inside comment
  #    blocks, in attribute values and possibly in other places.
  #    A better syntax is definitely needed if I want tags to
  #    parse as well-formed XML in all circumstances, but so far
  #    I haven't been able to devise a general solution. At the
  #    moment I recommend that the canonical form be used.
  #    The XML form is experimental at best, will definitely change
  #    in the future and may even be removed altogether, so don't use it.

  retago = retago "[-A-Za-z0-9_/.:]+([ \t]+enc *= *\".\")?" retagc

  if (!lvl) {				# only if 1st recursion.

     _TBLS[0,0] = split(tbl,tb,",")

     for (i=1; i<=_TBLS[0,0]; i++) {

       if (tb[i] !~ /^\//) tb[i] = "/dev/null"		# full-path ?

       # Load extra NoSQL tables if requested. Table rows will be assigned
       # to _TBLS[1], _TBLS[2], etc., with _TBLS[0,0] being the no. of tables
       # to load and _TBLS[n,0] is the no. of records in table 'n'. Any table
       # header information is stored in the separate uni-dimensional array
       # _TBLSH[]. For backward compatibility, the first (or only) of the
       # specified tables is loaded also in the _TBL[] uni-dimensional array,
       # and the relevant table header is stored in the _TBLH scalar.
       # Note that splitting table records into fields and unescaping NoSQL
       # special sequences is up to whoever will make use of these values,
       # often through the application-level _userproc() function.

       if (tb[i] != "") {
	  _TBLS[i,0] = 0
	  if (i==1) _TBL[0] = 0				# compatibility
	  while (getline record < tb[i] > 0) {
	     # Store table header in a separate place.
	     if (record ~ /^\001/ && _TBLSH[i] == _NULL) {
		_TBLSH[i] = record
		if (i==1) _TBLH = record		# compatibility
		continue
	     }
	     # Append raw table record to _TBLS[n].
	     _TBLS[i,++_TBLS[i,0]] = record
	     if (i==1) _TBL[++_TBL[0]] = record		# compatibility
	  }
	  close(tb[i])
       }
     }

     # This is mostly for 'csaExit.value'.

     if (ENVIRON["CSA_RPC_RESULT"] != _NULL) {
	# remove special value set by 'csaExit.value', if any.
	sub(/<csa:nil *\/>/,_NULL,ENVIRON["CSA_RPC_RESULT"])
	_OUT[0] = 1
	_OUT[1] = ENVIRON["CSA_RPC_RESULT"]
	gsub(/\t/,"\\t",_OUT[1])
	gsub(/\n/,"\\n",_OUT[1])
	gsub(/\001/,"\t",_OUT[1])

	_tbltordf(_OUTH,_OUT)

	# restore special pseudo-null value after calling _tbltordf().
	if (ENVIRON["CSA_RPC_RESULT"] == _NULL)
	   ENVIRON["CSA_RPC_RESULT"] = "<csa:nil />"

	# Mark the end of HTTP headers. Before this is done,
	# we may want to add further headers beside the basic
	# ones already printed by 'csaTemplate'.

	print _NULL >> ofile
     }

     for (j in ENVIRON) {

         # Always skip rc(1) functions, confidential data
         # and other CSA-specific stuff.

         if (j ~ ENVIRON["CSA_TPL_DENY"]) {
	    ENVIRON[j] = "<!-- removed -->"
	    continue
         }

         # If CSA_AUTH_SSL is in effect, let's be careful
         # about including HTTP_AUTHORIZATION in templates.

         if (j == "HTTP_AUTHORIZATION" && ENVIRON["CSA_AUTH_SSL"]) {
	      if (ENVIRON["SSL_CIPHER_USEKEYSIZE"] == "" && \
					ENVIRON["HTTPS"] != "on") {
	         # This cannot be set to anything other than "",
	         # or there will be problems with csaEnvtoxml.awk.
	         ENVIRON[j] = ""
	         continue
	      }
         }

	 # Only CSA_xxx variables are automatically encoded, while all
	 # others are left up to the application-level processor.
	 # Note that this is left solely for backwad compatibility, as
	 # with the new encoding flags it is no longer necessary.

	 if (j ~ /^CSA_/) _response(j,ENVIRON[j])
     }

     # Test whether the write-back file already contains something.
     getline input < ENVIRON["CSA_TPLWB"]
     close(ENVIRON["CSA_TPLWB"])

     if (input != _NULL) {

	# if the write-back file is not empty it means that we are
	# already executing its content. To prevent csaTemplate()
	# from looping on '. $CSA_TPLWB' we must therefore clear the
	# file and make sure nothing else is written to it.

	_creat(ofile,_O_TRUNC)
	ENVIRON["CSA_TPLWB"] = "/dev/null"
     }

     # safety valve, to prevent the whole movie to loop forever in
     # case other instructions that cause further loops through
     # the template processing code are written to CSA_TPLWB
     # (like 'csaExit.ok' and similar things).

     else print "CSA_TPLWB=/dev/null" > ENVIRON["CSA_TPLWB"]

     # Call the application-level processor, unless 'csaExit.value'
     # was called (to return only one, selected value).

     if (ENVIRON["CSA_RPC_RESULT"] == _NULL) {
	_userproc(_O_RESPONSE)

	# close the MRW stream, if any.
	if (_MRW["root"] != _NULL) _mrwresponse(".")

	# PRG is effective only if _userproc() did not write
	# different csaPrintMsg/csaExit.* directives to CSA_TPLWB.
	# Note that in case of PRG then we assume that if _userproc()
	# did not write earlier csaExit directives into CSA_TPLWB
	# we can safely run csaCommit. The latter must simply behave
	# as a no-op if there's nothing to commit, or if it was already
	# called by _userproc().

	if (prg != _NULL) {
	   if (_csa("idempotent") == _FALSE && _csa("confirmed") == _TRUE) {
	      _sys("csaCommit || csaExit.fault")
	   }
	   _sys("csaPrintMsg $CSA_MSG_OK")
	   _sys("csaExit.location " prg)
	}

	# handle template nullify requests issued by _sys(),
	# if called by _userproc() while processing responses.
	# Note that template nullification requests are taken into
	# consideration only if this is the first run.

	if (ENVIRON["CSA_TPLWB"] != "/dev/null" && \
		_RESULT == _ENOMSG) tpl = "/dev/null"
	else {
	   _tbltordf(_OUTH,_OUT)

	   # Mark the end of HTTP headers. Before this is done,
	   # we may want to add further headers beside the basic
	   # ones already printed by 'csaTemplate'.

	   print _NULL >> ofile
	}
     }
  }

  if (eval != "") split(eval,a,",")

  # Process the XML template file.

  while (getline input < tpl > 0) {
     nl = 1
     do {
        match(input,retago)
        if (RSTART) {
	   end = index(input, tagc)

	   # account for an "orphan" tagc before tago,
	   # i.e. "...] something $[....] something else ..."

	   if (end && end < RSTART) {
	      _rdfprint(substr(input, 1, end),parse,ofile)
	      input = substr(input, end + 1)
	      continue
	   }

	   tlen = end - RSTART - ltago
	   start = RSTART		# save before calling _rdfprint()
	   _rdfprint(substr(input, 1, start - 1),parse,ofile)
	   tag = substr(input, start + ltago, tlen)

	   # Turn XML encoding attribute into canonical form.
	   sub(/[ \t]+enc *= *"/,":",tag); sub(/"/,_NULL,tag)

	   # Extract encoding flags, if any.

	   if (tag ~ /:[a-zA-Z]+$/) {
	      enc = tag
	      sub(/.*:/,_NULL,enc)
	      sub(/:[a-zA-Z]+$/,_NULL,tag)
	   }

	   else enc = _NULL			# mandatory!

	   for (i in a) {			# Is "tag" to be eval'ed ?
	       if (a[i] == tag) {
		  t = T = tag
		  gsub(/\./,"__2e",t)		# Allow rc(1) encoded dots.
		  tag = ENVIRON[t]		# Get tag value from env.

		  # Get also new encoding flag from env, if available,
		  # otherwise stick with whatever encoding was defined
		  # in the original tag.

		  if (tag ~ /:[a-zA-Z]+$/) {
		     enc = tag
		     sub(/.*:/,_NULL,enc)
		     sub(/:[a-zA-Z]+$/,_NULL,tag)
		  }

		  # Evaluation of non-file tags is currently limited
		  # to the encoding flags. If this is not a file-tag,
		  # then restore the original file name and set the
		  # new value to the old one, but without the flags.

		  if (tag !~ /^-?\.?\//) {
		     ENVIRON[t] = tag
		     tag = T
		  }

		  break
	       }
	   }

	   if (tag ~ /^-?\.?\//) {		# Catch file patterns.
	      if (tag ~ /\.\./) {		# Catch any '..' pattern.
		 _rdfprint(tago tag tagc,parse,ofile) # Print tag literally.
	      }
	      else {

		 # Switch to no-replace mode if file tag begins with "-"
		 if (sub(/^-/,_NULL,tag)) norepl = 1

		 # Read external file, if it exists.
		 if (froot != _NULL && tag !~ /^\//) {
		    tag = froot "/" tag
		    gsub(/\/\//, "/", tag)	# Turn any "//" into "/".
		 }

		 # Only one single encoding flag, if any, is accepted
		 # in case of file tag.

		 if (enc == "c") {
		    # embeddable character-data, include verbatim.
		    _envtoxml(tag,froot,eval,tbl,trim,\
				tago,tagc,ofile,lvl+1,0,norepl,prg)
		 }
		 else if (enc == "p") {
		    # parsed-character data, whatever "parsed" means in
		    # the specific context (XML in this case).
		    _envtoxml(tag,froot,eval,tbl,trim,\
				tago,tagc,ofile,lvl+1,1,norepl,prg)
		 }
		 else {

		    # Default encoding for files is binary (Base64),
		    # possibly with further URI encoding, to make it
		    # suitable as an argument for GET type URLs.

		    if (enc != "B") enc = "b"

		    if (ofile != _STDOUT) close(ofile)
		    system("exec csa-encdec -e -" enc " < " tag " >> " ofile)
		    if (ofile != _STDOUT) _creat(ofile,_O_APPEND)
		 }
		 nl = norepl = 0
	      }
	   }
	   else {

	      if (norepl) value = tago tag tagc
	      else {
		 t = tag
		 # un-encode rc(1) dots, in case tag was eval'ed.
		 gsub("__2e",".",t)
		 if (t ~ ENVIRON["CSA_TPL_ALLOW"]) {
		    # Allow rc(1) encoded dots in tag name.
		    gsub(/\./,"__2e",tag)
		    value = ENVIRON[tag]
		 }
		 else value = "<!-- removed -->"
	      }

	      # Perform encoding, if requested.

	      i = split(enc,e,_NULL)
	      for (j=1; j<=i; j++) {
	          if (e[j] == "i") value = _isoencode(value)
	          else if (e[j] == "j") value = _jsencode(value)
	          else if (e[j] == "u") value = _uriencode(value)
	          else if (e[j] == "x") value = _xmlencode(value)
	          else if (e[j] == "P") value = _uriencode(value,1)
	      }
	      enc = _NULL			# clear for next loop!
	      _rdfprint(value,parse,ofile)
	   }

	   input = substr(input, end + ltagc)
	}
	else {
	   _rdfprint(input,parse,ofile)
	   input = _NULL		# Not to loop forever.
	}
     }  while (length(input))

     if (nl) _rdfprint("\n",0,ofile)
  }

  close(tpl)
}

