# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2003,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# void _rwa2csa(string s)
#
# Extract the RwA (RDF with Attachments) XML payload from an RwA MIME
# multipart request. The extracted values are inserted into the global
# _CGIVAL[] array. This function must be called *after* the multipart
# input has been parsed, and the relevant binary objects have been
# saved.
# =====================================================================

function _rwa2csa(s,		a,i,j,found,level,type,name,\
				data,num,fmt,rdfmatch) {

  i = split(s, a, "<")

  for (j=1; j<=i; j++) {

    # detect <rdf:RDF ...>
    if (a[j] ~ /^rdf:RDF[ \t>]/) {found=level=1; continue}

    if (!found) continue

    # detect <... rdf:about="#call">
    rdfmatch = a[j]
    sub(/[\t ]+rdf:about="#call">.*/,"",rdfmatch)
    rdfmatch = "^/" _escreg(rdfmatch) ">"

    if (level==1 && \
	a[j] ~ /^[-a-zA-Z0-9:]+[\t ]+rdf:about="#call">/) {

      level++
      continue
    }

    # detect <param>
    if (level==2 && a[j] ~ /^(csa:)?param>/) {level++; continue}

    # Extract call arguments.
    if (level==3 && a[j] ~ /^rdf:Seq[\t ]+rdf:about="#call\./) {

       sub(/^.*rdf:about="#call\./,"",a[j]); sub(/".*/,"",a[j])

       # replace invalid characters with underscores (_) in name.
       gsub(/[^A-Za-z0-9_]/, "_", a[j])

       # always skip call.0 in call arguments.
       if (a[j] != "0") name = a[j]

       level++
       continue
    }

    # detect <rdf:li[^=]+rdf:datatype="...> in request and extract datatype
    if (level==4 && name != "" && \
		a[j] ~ /^rdf:li[^=]+rdf:datatype="/) {

       # Support three common forms of schema reference.
       if (sub(/^[^>]*XMLSchema#/,"",a[j]));
       else if (sub(/^[^>]*&xsd;/,"",a[j]));
       else sub(/^[^>]*xsd:/,"",a[j])

       type = a[j]; sub(/".*/,"",type)

       # Unknown types default to "string".

       if (type !~ /^(int|base64Binary|boolean|double)$/) type = "string"

       # detect empty tag.
       if (a[j] ~ /[^>]*\/>/) a[j] = ""
       else {
          sub(/[^>]*>/,"",a[j])

          if (type ~ /^(int|double)$/) a[j] += 0	# cast to numeric

	  if (type == "base64Binary") {

	     _cgivar(name,\
	       ENVIRON["TMPDIR"] "/hexBinary." _PID "." name ".tmp",1)

	     if (_CMD != "") close(_CMD)		# just in case
	     _CMD = "exec csa-encdec -d -b > " _CGIVAL[name]
	     printf("%s", a[j]) | _CMD
	     close(_CMD)
	  }

	  # add any type-specific checks on a[j] here ...
          else _cgivar(name,_xmldecode(a[j]))
       }

       continue
     }

     # detect <rdf:li rdf:resource=".../> in request and extract the
     # associated URI, if valid.

     if (level==4 && name != "" && a[j] ~ /^rdf:li[^=]+rdf:resource="/) {

	sub(/[^=]+resource="/,"",a[j]); sub(/".*/,"",a[j])

	# skip invalid URI's.

	if (_isuri(a[j]) == _TRUE) _cgivar(name, a[j], 2)

	continue
     }

     # detect <rdf:li rdf:about=".../> in request and detect whether
     # they refer to RwA attachments.

     if (level==4 && name != "" && a[j] ~ /^rdf:li[^=]+rdf:about="/) {

	sub(/[^=]+about="/,"",a[j]); sub(/".*/,"",a[j])


	# Resolve RwA attachments. See doc/rquest.rwa for why I'm
	# not using the "file:" URI scheme here.

	if (sub(/^#attachment[:.]/,"",a[j])) {

	   # replace invalid characters with underscores (_) in a[j]
	   # (the same is done by csaCgi.awk when searching the input
	   # payload for attachment names).
	   gsub(/[^A-Za-z0-9_]/, "_", a[j])

	   _cgivar(name,\
	       ENVIRON["TMPDIR"] "/hexBinary." _PID "." a[j] ".tmp",1,1)
	}

	# ignore non-attachment 'rdf:about' defs at level 4.
	continue
     }

     # check that peer is declaring a supported CSA-RPC format.
     if (level==2 && a[j] ~ /^dc:format>/) {

	sub(/^dc:format>/,"",a[j]); sub(/[^0-9.]/,"",a[j]); fmt = a[j]
     }

     # detect </rdf:li>
     if (level==4 && a[j] ~ /^\/rdf:li>/) continue

     # detect </rdf:Seq>
     if (level==4 && a[j] ~ /^\/rdf:Seq>/) {
	name = ""
	level--
	continue
     }

     # detect </param>
     if (level==3 && a[j] ~ /^\/(csa:)?param>/) {
	level--
	continue
     }

     # detect </rdf:Description> or whatever we called the outer element.
     if (level==2 && a[j] ~ rdfmatch ) {
	level--
	continue
     }

     if (level==1 && a[j] ~ /^\/rdf:RDF>/) {level--; return}
  }

  if (!found || level) return 150	# No RDF, or parse error.

  # Update this test as the CSA-RPC format evolves.
  if (fmt"" != "0.1") return 151	# Unsupported format.
}

