# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _tbltordf.awk,v 1.3 2006/03/10 21:36:17 carlo Exp $
# =====================================================================
# int _tbltordf(string tblh, array tblb [, boolean trim, string body])
#
# Turn the NoSQL structure tblh/tblb table into a proper RDF structure.
# Although it would be possible to preserve the table data dictionary
# (i.e. column names) on output, I have deliberately choosen not to
# do so, to stick to the view that an RPC mechanism should return
# unnamed values (possibly arrays). The data in 'tblb' does not need
# to be in "canonical" table format, that is short/long records are
# acceptable. This was done on purpose, to make it possible to return
# varable-length arrays in RPC responses.
# =====================================================================

function _tbltordf(tblh,tblb,trim,body,	      c_names,isuri,\
						c_values,nr,nf,i) {
  # set defaults.
  if (body == "") body = ENVIRON["CSA_RDF_BODY"]


  # I now accept also header-less tables.
  #
  #if (!gsub(/\001/,"",tblh)) return 1	# not a NoSQL table
  #nf = split(tblh,c_names,"\t")

  # return an error if no tab-separated fields are found.
  #if (!(nf=split(tblb[1],c_values,"\t"))) return 1

  for (nr=1; nr <= tblb[0]; nr++) {		# for each record

      printf("<result>\n<rdf:Seq rdf:about=\"#response.%d\">\n", nr) > body

      # Set a new 'nf' value upon each record, to support
      # variable-length response arrays.

      nf = split(tblb[nr], c_values, "\t")

      for (i=1; i <= nf; i++) {		# process each field in turn.

	  printf("<rdf:li rdf:") > body

	  # Try to figure-out whether it is a resource or a literal.

	  if (_isuri(tolower(c_values[i])) == _TRUE) {
	     isuri = 1
	     printf("resource=\"") > body
	  }
	  else {
	     isuri = 0
	     printf("datatype=\"") > body
	  }

	  if (trim) {
	     sub(/^[ \t]+/,"",c_values[i])
	     sub(/[ \t]+$/,c_values[i])
	  }

	  if (isuri)
	     printf("%s\" />\n", _xmlencode(_unesctbl(c_values[i]))) > body
	  else printf("%s\">%s</rdf:li>\n", _rdftype(c_values[i]),\
				_xmlencode(_unesctbl(c_values[i]))) > body
      }

      printf("</rdf:Seq></result>\n") > body
  }

  close(body)
}

